@@ -6,17 +6,34 @@ import json
 import threading
 import time
 from pathlib import Path
-
 import datetime
+import typing
 
 
-def check_dict_scheme(d, s, return_bool=False):
+def check_dict_scheme(
+    d: dict, s: dict, return_bool: bool = False
+) -> typing.Union[None, bool]:
+    """compares dict d to given scheme s
+
+    e.g.:
+
+    d = {"measurement":{"rpm_min":16.0}}
+    s = {"measurement":{"rpm_min":((int,float), 15, 17)}} <-- rpm_min is int or float and lies inbetween 15 and 17
+    print(check_dict_scheme(d,s,True))
+    >> True
+
+    s = {"measurement":{"rpm_min":((int,float), 17, 18)}} <-- rpm_min is int or float and lies inbetween 17 and 18
+    print(check_dict_scheme(d,s,True))
+    >> False
+
+    !! caution 16 != 16.0 --> int <-> float --> use
+    """
     result = True
     for k, v in d.items():
         if isinstance(v, dict):
             if s[k] and isinstance(s[k], dict):
                 if return_bool:
-                    result &= check_dict_scheme(v, s[k], True)
+                    result &= bool(check_dict_scheme(d=v, s=s[k], return_bool=True))
                 else:
                     check_dict_scheme(v, s[k], False)
             elif not isinstance(s[k], dict):
@@ -25,8 +42,11 @@ def check_dict_scheme(d, s, return_bool=False):
                 else:
                     raise ValueError(f"Wrong Scheme! {k} is dict instead of {s[k][0]}")
         else:
-            if k not in s:
-                raise ValueError(f"{k} not found in schmeme, check spelling")
+            if k not in s.keys():
+                if return_bool:
+                    return False
+                else:
+                    raise ValueError(f"{k} not found in scheme, check spelling")
             tuple_comp = s[k]
             if not isinstance(v, tuple_comp[0]):
                 if return_bool:
@@ -35,7 +55,7 @@ def check_dict_scheme(d, s, return_bool=False):
                     f"wrong type given for {k}: {type(v)} instead of {tuple_comp[0]}"
                 )
 
-            if tuple_comp[0] in (int, float):
+            if tuple_comp[0] in (int, float, (int, float)):
                 if not (tuple_comp[1] <= v <= tuple_comp[2]):
                     if return_bool:
                         return False
@@ -51,21 +71,41 @@ def check_dict_scheme(d, s, return_bool=False):
                     )
             elif tuple_comp[0] in (bool,):
                 pass
+            else:
+                raise SyntaxError("Cant Validate input")
     if return_bool:
         return result
 
 
-def wait_until(next_time, delta):
+def wait_until(
+    now: typing.Union[float, int],
+    delta: typing.Union[float, int],
+    ensure_mod: bool = False,
+) -> typing.Union[float, int]:
+    """
+    allows easy regular execution of following code when used as following:
+    t = time.time()
+    while True:
+        t = wait_until(t, 10)
+        print(time.time())
+    """
+    if delta < 0:
+        return now
 
+    next_time = now + delta
     while next_time - time.time() > 0:
         time.sleep(0.75 * (next_time - time.time()))
 
-    return next_time + delta
+    if ensure_mod:
+        next_time = int(next_time)
+        while next_time % delta != 0:
+            next_time += 1
+    return next_time
 
 
 class NanoVib:
-    def __init__(self) -> None:
-        self._base_url = "http://192.168.4.1"
+    def __init__(self, ip: str = "192.168.4.1"):
+        self._base_url = f"http://{ip}"
         self._ensure_connection()
         self.settings = requests.get(self._base_url + "/settings").json()
         self._settings_scheme = {
@@ -87,7 +127,7 @@ class NanoVib:
                 "rpm_min": (float, 0, 2e32),
                 "rpm_max": (float, 0, 2e32),
                 "measurement_time": (int, 1000, 300),
-                "folder": "Test_Background_App",
+                "folder": (str, 3, 254),
                 "parted": (bool,),
                 "part_size": (int, 1024 * 1024, 2e32),
                 "vibA_compatibility": (bool,),
@@ -106,32 +146,37 @@ class NanoVib:
         self.status_thread = threading.Thread(target=self._refresh_status, daemon=True)
         self.status_thread.start()
 
-    def _ensure_connection(self):
+    def _ensure_connection(self) -> None:
         connected = False
-        for _ in range(10):
+        for i in range(10):
             try:
                 self.status = requests.get(self._base_url + "/status", timeout=3).json()
                 connected = True
                 break
             except Exception as e:
-                warnings.warn("Connection takes longer than expected")
+                if i > 3:
+                    warnings.warn("Connection takes longer than expected")
+                    warnings.warn(str(e))
 
         if not connected:
             raise ConnectionError(
                 "Could not connect to NanoVib, ensure Wifi Connection to NanoVib and try again"
             )
 
-    def _update_time(self):
+    def _update_time(self) -> None:
         tz = pytz.timezone("Europe/Berlin")
         now = datetime.datetime.now().astimezone(tz)
-        patch_dict = {
-            "system": {
-                "epoch": int(now.timestamp()) + int(now.utcoffset().total_seconds())
-            }
-        }
+        offset = now.utcoffset()
+
+        if isinstance(offset, datetime.timedelta):
+            offset_seconds = offset.total_seconds()
+        else:
+            offset_seconds = 0
+
+        patch_dict = {"system": {"epoch": int(now.timestamp() + offset_seconds)}}
         self._update_status(patch_dict)
 
-    def _refresh_status(self):
+    def _refresh_status(self) -> None:
         t_next = int(time.time()) + 2
         while True:
             try:
@@ -139,15 +184,15 @@ class NanoVib:
             except Exception as e:
                 print(e)
 
-            t_next = wait_until(t_next, 2)
+            t_next = wait_until(now=t_next, delta=2)
 
-    def _update_status(self, patch_dict):
+    def _update_status(self, patch_dict) -> None:
         check_dict_scheme(patch_dict, self._status_scheme)
         self.status = requests.patch(
             self._base_url + "/status", data=json.dumps(patch_dict)
         ).json()
 
-    def start_measurement(self, max_revs=None, measurement_time_min=None):
+    def start_measurement(self, max_revs=None, measurement_time_min=None) -> None:
         settings_dict = {
             "measurement": {
                 "max_revolutions": (2e32 if max_revs == None else int(max_revs)),
@@ -163,17 +208,19 @@ class NanoVib:
         patch_dict = {"measurement": {"recording": True}}
         self._update_status(patch_dict)
 
-    def stop_measurement(self):
+    def stop_measurement(self) -> None:
         patch_dict = {"measurement": {"recording": False}}
         self._update_status(patch_dict)
 
-    def list_measurements(self):
+    def list_measurements(self) -> typing.Tuple[bool, typing.List[dict]]:
         if self.status["measurement"]["recording"]:
             return (False, self.measurements)
         else:
             return (True, requests.get(self._base_url + "/measurements").json())
 
-    def get_measurement(self, project=None, measurement_tag=None, generator=False):
+    def get_measurement(
+        self, project=None, measurement_tag=None, generator=False
+    ) -> typing.Generator[Path, None, None]:
         self._ensure_connection()
 
         """ determine wether live measurement -> live download or project and measurement are given -> only if not measuring """
@@ -189,12 +236,15 @@ class NanoVib:
             ]
         else:
             if self.status["measurement"]["recording"]:
-                warnings.warn("measurements active! Download of measurement during another active measurement is not recommended!")
+                warnings.warn(
+                    "measurements active! Download of measurement during another active measurement is not recommended!"
+                )
             if project:
                 patch_dict = {"measurement": {"folder": project}}
                 self.update_settings(patch_dict)
             measurements = requests.get(self._base_url + "/measurements").json()
             found = False
+            m = {"name": "", "size": 0}
             for m in measurements:
                 if measurement_tag in m["name"]:
                     found = True
@@ -203,10 +253,8 @@ class NanoVib:
                 raise ValueError("Measurement not found")
             meas_base = m["name"]
 
-
-
         if meas_base.endswith(".txt") or meas_base.endswith(".csv"):
-            """not parted -> download and if generator yield else return"""
+            """not parted -> download and if generator yield else return TODO"""
             pass
 
         else:
@@ -224,7 +272,7 @@ class NanoVib:
 
             while True:
 
-                """ find highest found partnumber """
+                """find highest partnumber of meas"""
                 part_found_max = 0
                 for p in [p for p in dl_path.iterdir() if meas_tag in p.name]:
                     if p.stem.startswith(meas_tag):
@@ -237,7 +285,11 @@ class NanoVib:
 
                 """ generate url and path of parted file """
                 part_url = (
-                    self._base_url + meas_base +"/"+ str(part_found_max + 1).zfill(4) + ext
+                    self._base_url
+                    + meas_base
+                    + "/"
+                    + str(part_found_max + 1).zfill(4)
+                    + ext
                 )
                 part_file = dl_path / (
                     meas_tag + "_" + str(part_found_max + 1).zfill(4) + ext
@@ -251,8 +303,6 @@ class NanoVib:
                     time.sleep(5)
                     continue
 
-                
-
                 if resp.status_code == 200:
                     with open(part_file, "wb") as f:
                         f.write(resp.content)
@@ -261,15 +311,14 @@ class NanoVib:
                         f.write(resp.content)
 
                 if resp.status_code == 409:
-                    if generator and last_yielded_at_part != part_found_max:
+                    if last_yielded_at_part != part_found_max:
                         last_yielded_at_part = part_found_max
+                        t0 = time.time()
                         yield combined_file
-                    else:
-                        time.sleep(5)
+                        wait_until(now=t0, delta=5)
 
                 if resp.status_code == 404:
-                    if generator and last_yielded_at_part != part_found_max:
-                        last_yielded_at_part = part_found_max
+                    if last_yielded_at_part != part_found_max:
                         yield combined_file
                     break
 
@@ -282,7 +331,9 @@ class NanoVib:
             self._base_url + "/settings", data=json.dumps(patch_dict), timeout=1
         )
         if resp.status_code != 200:
-            warnings.warn(resp.status_code)
+            warnings.warn(
+                f"Status Code 200 expected, got {resp.status_code} - {resp.text}"
+            )
         else:
             self.settings = resp.json()
 
